/*
INCLUDE -- select file entries from a list
    Copyright (C) 2003  Norm London, London Computing

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	norm_london@londoncomputing.com

    Compile with Visual Studio 6 or later. cl include.c
    Compile with GCC 2.95.2 or later	   gcc -oinclude  (windows and linux)
    If compiling on MAC SYSTEM X (and possibly other BSD based os's), you will
    need to modify the include for malloc.h to sys/malloc.h

*/



#include <stdio.h>
#include <string.h>
#include <malloc.h> 
#ifdef WIN32
#include <process.h> 
#else
#define stricmp strcasecmp
#define strnicmp strncasecmp
#endif

char s[32767];

char *stristr(char *s,char *t) {
	int i,ls,lt;

	ls = strlen(s);
	lt = strlen(t);


	for (i=0; i <= (ls - lt); ++i)
		if (strnicmp(s+i,t,lt) == 0)
			return s+i;

	return NULL;
}

void help() {	
FILE *hlp = stderr;
fprintf(hlp,"INCLUDE Version 1.0, %s\n",__DATE__);
fprintf(hlp,"\nCopyright 2003, Norm London, London Computing.\nnorm_london@londoncomputing.com\n\n");
fprintf(hlp,"This program is free software; you can redistribute it and/or modify\n");
fprintf(hlp,"it under the terms of the GNU General Public License as published by\n");
fprintf(hlp,"the Free Software Foundation; either version 2 of the License, or\n");
fprintf(hlp,"(at your option) any later version.\n\n");
fprintf(hlp,"This program is distributed in the hope that it will be useful,\n");
fprintf(hlp,"but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
fprintf(hlp,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
fprintf(hlp,"GNU General Public License for more details.\n\n");
fprintf(hlp,"You should have received a copy of the GNU General Public License\n");
fprintf(hlp,"along with this program; if not, write to the Free Software\n");
fprintf(hlp,"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");


fprintf(stderr,"Usage is: INCLUDE [{[-U] [-S]|-?}] filter.dat <infile >outfile\n");
fprintf(stderr,"Copies standard input to standard output while applying filters\n");
fprintf(stderr,"from filter.dat.  If any line in filter.dat is a sub-string of\n");
fprintf(stderr,"a line read from stdin, that line is copied to stdout.\n");
fprintf(stderr,"Returns 0 if good execution, 4 if this help is displayed \n");
fprintf(stderr,"or an error occurs.\n\n");
fprintf(stderr,"The -U flag indicates that case should be ignored.\n");
fprintf(stderr,"    -? flag prints this help.\n");
fprintf(stderr,"    -S removes an entire section if a filter expression\n");
fprintf(stderr,"       is found within a Windows .INI or .REG file, i.e.\n");
fprintf(stderr,"       within [ ... ]\n");
}

int main(int argc,char *argv[]) {
    /* Usage is: INCLUDE [-U] [-S] [filter] <file1 >file2 */
	/* Copies stdin to stdout, excluding those records lines containing any */
	/* of the exclusion terms stored in filter. */
	/* -U -- ignore case */
    /* -S -- Remove entire section if filter expression is found within [ ... ]*/
	struct flist {
		struct flist *next;
		char *exp;
	} 
	*ff,*cf, *mf;

	FILE *filter;

	int arg,len;
	char ignore_case = 0, *found;
    	char delete_section = 0, skipping_section = 0;
	long inkt=0, otkt = 0;

	ff = NULL;
	cf = ff;
	filter = NULL;

	for (arg=1;arg < argc; ++arg) {
		if (stricmp(argv[arg],"?") == 0 || stricmp(argv[arg],"-?") == 0) {
			help();
			exit(4);
		}
		if (stricmp(argv[arg],"-U") == 0)
			ignore_case = 1;
        else if (stricmp(argv[arg],"-S") == 0)
            delete_section = 1;
		else {
			filter = fopen(argv[arg],"r");
			if (filter == NULL) 
				fprintf(stderr,"Unable to open: %s\n",argv[arg]);
			else {
				fprintf(stderr,"Reading filters from: %s\n",argv[arg]);
				while (fgets(s,sizeof(s),filter) != NULL) {
					if (s[0] == '\n')
						continue;
					mf = (struct flist *) malloc(sizeof(struct flist));
					if (mf == NULL) {
						fprintf(stderr,"Out of Memory\n");
						exit(4);
					}
					if (ff == NULL)
						ff = mf;
					else
						cf->next = mf;

					cf = mf;
					cf->exp = (char *) malloc(len=strlen(s));
					cf->next = NULL;
					if (cf->exp == NULL) {
						fprintf(stderr,"Out of Memory\n");
						exit(4);
					}
					if (s[len-1] == '\n')
						s[len-1] = 0; /* Remove trailing line feeds */
/*					if (ignore_case) strupr(s); */
					strcpy(cf->exp,s);
					/*	fprintf(stderr,"ff=%4x, cf=%x, cf->exp=%s, cf->next=%x\n",ff,cf,cf->exp,cf->next); */
				}
			}
		}
	}

	if (filter != NULL)
		fclose(filter);

/*	fprintf(stderr,"Excluding files containing the following:\n"); 
  	for (cf=ff;cf != NULL;cf = cf->next)
		fprintf(stderr,"ff=%4x, cf=%x, cf->exp=%s, cf->next=%x\n",
		ff,cf,cf->exp,cf->next);
*/

	/* The list of filter statments has been built, now do the copy */
	while(fgets(s,sizeof(s),stdin) != NULL) {
		++inkt;
		/* read in a line, apply filter, write out */
		/* if (ignore_case)	strupr(s); */
        if (skipping_section)
            if (s[0] == '[')
                skipping_section = 0;
            else
                continue;

		for (cf=ff;cf != NULL; cf = cf->next) {
			if ((found = stristr(s,cf->exp)) != NULL)
				break;
		}		
		if (found != NULL) {
			++otkt;
			fputs(s,stdout);
		}
        else if ((s[0] == '[') && delete_section)
            skipping_section = 1;

	}

	cf = ff;
	while (cf != NULL) {
		ff = cf;
		cf = cf->next;
		free(ff);
	}

	fprintf(stderr,"%ld records read, %ld records written.\n",inkt,otkt);
	
	
	exit(0);
}

			
				
		
